#define FIND_COMMON_POINT	0

#include <errno.h>
#include <poll.h> // for poll()
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "meye_control.h"
#include "camcap.h"
#include "brview.h"
#include "obrec.h"

int stopcapturing = 0;

void iseyes_rectangular_pan(meyec_t *ec);
void iseyes_random_pans(meyec_t *ec);

int sockfd;
unsigned char *shmptr;
char *shmname = "/dnshm";
char *sockname = "/dnserver";

/*
 * YUV_to_Bitmap
 * adapted from: http://paulbourke.net/dataformats/yuv/
*/
static void YUV_to_Bitmap(unsigned char *i, int y, int u, int v)
{
   int r,g,b;

   // u and v are +-0.5
   u -= 128;
   v -= 128;

   // Conversion
   r = y + 1.370705 * v;
   g = y - 0.698001 * v - 0.337633 * u;
   b = y + 1.732446 * u;

/*
   r = y + 1.402 * v;
   g = y - 0.344 * u - 0.714 * v;
   b = y + 1.772 * u;
*/
/*
   y -= 16;
   r = 1.164 * y + 1.596 * v;
   g = 1.164 * y - 0.392 * u - 0.813 * v;
   b = 1.164 * y + 2.017 * u;
*/

   // Clamp to 0..1
   if (r < 0) r = 0;
   if (g < 0) g = 0;
   if (b < 0) b = 0;
   if (r > 255) r = 255;
   if (g > 255) g = 255;
   if (b > 255) b = 255;

   i[2] = (unsigned char) r;
   i[1] = (unsigned char) g;
   i[0] = (unsigned char) b;
   i[3] = '\0';

   return;
}

#if 0
// resizing code copied from darknet code
typedef struct {
    int w;
    int h;
    int c;
    float *data;
} image;

static float get_pixel(image m, int x, int y, int c)
{
    assert(x < m.w && y < m.h && c < m.c);
    return m.data[c*m.h*m.w + y*m.w + x];
}

static void set_pixel(image m, int x, int y, int c, float val)
{
    if (x < 0 || y < 0 || c < 0 || x >= m.w || y >= m.h || c >= m.c) return;
    assert(x < m.w && y < m.h && c < m.c);
    m.data[c*m.h*m.w + y*m.w + x] = val;
}

static void add_pixel(image m, int x, int y, int c, float val)
{
    assert(x < m.w && y < m.h && c < m.c);
    m.data[c*m.h*m.w + y*m.w + x] += val;
}

image make_empty_image(int w, int h, int c)
{
    image out;
    out.data = 0;
    out.h = h;
    out.w = w;
    out.c = c;
    return out;
}

image make_image(int w, int h, int c)
{
    image out = make_empty_image(w,h,c);
    out.data = calloc(h*w*c, sizeof(float));
    return out;
}

int camcap_scale(unsigned char *dst, int dwid, int dhgt, int dstride, unsigned char *src, int swid, int shgt, int sstride)
{
image resize_image(image im, int w, int h)
{
    image resized = make_image(w, h, im.c);
    image part = make_image(w, im.h, im.c);
    int r, c, k;
    float w_scale = (float)(im.w - 1) / (w - 1);
    float h_scale = (float)(im.h - 1) / (h - 1);
    for(k = 0; k < im.c; ++k){
        for(r = 0; r < im.h; ++r){
            for(c = 0; c < w; ++c){
                float val = 0;
                if(c == w-1 || im.w == 1){
                    val = get_pixel(im, im.w-1, r, k);
                } else {
                    float sx = c*w_scale;
                    int ix = (int) sx;
                    float dx = sx - ix;
                    val = (1 - dx) * get_pixel(im, ix, r, k) + dx * get_pixel(im, ix+1, r, k);
                }
               set_pixel(part, c, r, k, val);
            }
        }
    }
    for(k = 0; k < im.c; ++k){
        for(r = 0; r < h; ++r){
            float sy = r*h_scale;
            int iy = (int) sy;
            float dy = sy - iy;
            for(c = 0; c < w; ++c){
                float val = (1-dy) * get_pixel(part, c, iy, k);
                set_pixel(resized, c, r, k, val);
            }
            if(r == h-1 || im.h == 1) continue;
            for(c = 0; c < w; ++c){
                float val = dy * get_pixel(part, c, iy+1, k);
                add_pixel(resized, c, r, k, val);
            }
        }
    }

    free_image(part);
    return resized;
}
#endif

/*
 * In V4L2_PIX_FMT_YUYV, 4 bytes is 2 pixels with shared information.
 * See:
 * https://v4l.videotechnology.com/dwg/v4l2.html#V4L2-PIX-FMT-YUYV
 * Note that I was initially getting green/violet colors. From:
 * https://stackoverflow.com/questions/16176415/v4l2-pix-fmt-yuyv-convert-from-yuyv-to-rgb24
 * this turned out to be an endian issue where the y1 and u and the y2 and v
 * were reversed.
*/
static void iseyes_convert_to_rgb(brview_image_t *img, camcap_t *cc)
{
	int x, y;
	unsigned char *i, *c;
	int y1, y2, u, v;

	c = cc->vb;
	for (y = 0; y < cc->height; y++) {
		i = &(img->data[y*img->stride]);
		for (x = 0; x < cc->width/2; x++) {
			// for each YUYV 4 bytes contain shared info for 2 image pixels
			u = c[1];
			y1 = c[0];
			v = c[3];
			y2 = c[2];
			YUV_to_Bitmap(i, y1, u, v);
			i += 4;
			YUV_to_Bitmap(i, y2, u, v);
			i += 4;
			c += 4; 
		}
	}

#if 0
// This was an initial attempt to try out scaling of the image using the
// Jetson TX1's GPU code.
//	if (img->width != cc->width || img->height != cc->height) {
		// scale the image
	unsigned char *s;
		if ((s = malloc(img->height*img->stride)) == NULL) {
			return;
		}
		if (camcap_scale(s, img->width, img->height, img->stride, c, cc->width, cc->height, cc->stride) == -1) {
			free(s);
			return;
		}
		memcpy(img->data, s, img->height*img->stride);
		free(s);
//	}
#endif
}

#if FIND_COMMON_POINT
// This approach that wasn't working. The distance to the object
// calculated was too short (1200mm instead of 2200mm). The plan was to
// calculate the distance and the angles from a point between the eyes
// and then move the eyes based on that.

// the following were measured by taking photos and making measurements
#define Rhfvnd	38.8	// right, horiz field-of-view number of degrees
#define Rvfvnd	29.7	// right eye, vert field-of-view number of degrees
#define Lhfvnd	37.3	// left eye, horiz field-of-view number of degrees
#define Lvfvnd	27.3	// left eye, vert field-of-view number of degrees
// calculations for the arc behind the flat image:
// circum = 2*M_PI*200mm
// length_of_arc = (Rhvnd/360) * circum
// pixels_for_arc = (ccr->width/measured_seen_image_width) * length_of_arc
static int iseyes_find_common_point(meyec_t *ec, objectr_t *or, objectr_t *ol, camcap_t *ccr, camcap_t *ccl)
{
	double Rsa1, Lsa1, Rsa2, Lsa2, hwa, hwd;
	// the image width can either be the actual flat width of the image
	// (e.g. 640) or the arc the image would form.
	//double rwid = ccr->width; // flat width in pixels
	//double lwid = ccl->width; // flat width in pixels
	// The radius of the arc is not 200, it's the length of the side of 
	// the triangle, which is gotten by the pythagorean theorem.
	double fvradius = sqrt(200*200+(141/2)*(141/2));
	double rwid = (ccr->width/141.0)*((Rhfvnd/360.0)*(2*M_PI*fvradius)); // arc width in pixels
	//double rwid = ((Rhfvnd/360)*(2*M_PI*fvradius)); // arc width in mm
	fvradius = sqrt(200*200+(135/2)*(135/2));
	double lwid = (ccl->width/135.0)*((Lhfvnd/360.0)*(2*M_PI*fvradius)); // arc width in pixels
	//double lwid = (Lhfvnd/360)*(2*M_PI*fvradius); // arc width in mm

	Rsa1 = ((or->left+((or->right-or->left)/2))*(Rhfvnd/rwid))+(180.0-Rhfvnd)/2;
	Lsa1 = ((ol->left+((ol->right-ol->left)/2))*(Lhfvnd/lwid))+(180.0-Lhfvnd)/2;
	Rsa2 = Rsa1-(ec->e[MEYE_R].horiz_cur-ec->e[MEYE_R].horiz_mid);
	Lsa2 = 180-((ec->e[MEYE_L].horiz_cur-ec->e[MEYE_L].horiz_mid)+Lsa1);
	hwa = 180-Rsa2-((180-Lsa2-Rsa2)/2);
	// used units of pixels above, we're using mm below
	hwd = sin(Rsa2*(M_PI/180))/(sin((180-Rsa2-hwa)*(M_PI/180))/meyec_get_eye_spacing_half());

	printf("or->left,right=%d, %d, Rhfvnd=%f, ccr->width=%d, rwid=%f\n",
			or->left, or->right, Rhfvnd, ccr->width, rwid);
	printf("ol->left,right=%d, %d, Lhfvnd=%f, ccl->width=%d, lwid=%f\n",
			ol->left, ol->right, Lhfvnd, ccl->width, lwid);
	printf("hwa=%f, hwd=%f, Rsa1=%f,Rsa2=%f,Lsa1=%f,Lsa2=%f\n",
		hwa, hwd, Rsa1, Rsa2, Lsa1, Lsa2);

	double RHa, RVa, LHa, LVa;
	
	static int first = 1;
	if (first) {
	//meyec_move_eyes_to_loc(ec, hwa, ec->e[MEYE_L].vert_mid/*Va*/, hwd);
	meyec_calc_eye_angles(ec, &RHa, &RVa, &LHa, &LVa, hwa, 0.0/*Va*/, hwd);
	printf("RHa=%f, RVa=%f, LHa=%f, LVa=%f\n", RHa, RVa, LHa, LVa);
	if (meyec_move_eye_angle(ec, MEYE_R, MEYE_H, RHa) == -1)
		return -1;
	if (meyec_move_eye_angle(ec, MEYE_R, MEYE_V, RVa) == -1)
		return -1;
	if (meyec_move_eye_angle(ec, MEYE_L, MEYE_H, LHa) == -1)
		return -1;
	if (meyec_move_eye_angle(ec, MEYE_L, MEYE_V, LVa) == -1)
		return -1;
	//stopcapturing = 1;
	//first = 0;
	//exit(EXIT_SUCCESS);
	}

	return 1;
}
#endif

static void iseyes_calc_distance_to_object(meyec_t *ec, objectr_t *or, objectr_t *ol, camcap_t *ccr, camcap_t *ccl)
{
	double Rsa2, Lsa2, Oa, Rd;

	printf("MEYE_R.horiz_cur=%d, horiz_mid=%d\n",
		ec->e[MEYE_R].horiz_cur, ec->e[MEYE_R].horiz_mid);
	printf("MEYE_L.horiz_cur=%d, horiz_mid=%d\n",
		ec->e[MEYE_L].horiz_cur, ec->e[MEYE_L].horiz_mid);
	printf("meyec_get_eye_spacing_half()=%f\n", meyec_get_eye_spacing_half());
	Rsa2 = 180-((ec->e[MEYE_R].horiz_cur-ec->e[MEYE_R].horiz_mid)+90);
	Lsa2 = (ec->e[MEYE_L].horiz_cur-ec->e[MEYE_L].horiz_mid)+90;
	Oa = 180-Lsa2-Rsa2;
	double sinLsa2;
	sinLsa2 = sin(Lsa2*(M_PI/180.0));
	printf("M_PI=%f, sin(Lsa2)=%f, sin(Oa)=%f\n", M_PI, sinLsa2, sin(Oa*(M_PI/180.0)));
	Rd = ((meyec_get_eye_spacing_half()*2)/sin(Oa*(M_PI/180.0)))*sin(Lsa2*(M_PI/180.0));
	printf("Rsa2=%f, Lsa2=%f, Rd=%f\n", Rsa2, Lsa2, Rd);
	static int first = 1;
	if (first) {
//	stopcapturing = 1;
	}
}

// the following were measured by taking photos and making measurements
#define Rhfvnd	40//38.8	// right, horiz field-of-view number of degrees
#define Rvfvnd	31//29.7	// right eye, vert field-of-view number of degrees
#define Lhfvnd	37.3	// left eye, horiz field-of-view number of degrees
#define Lvfvnd	27.3	// left eye, vert field-of-view number of degrees
// calculations for the arc behind the flat image:
// circum = 2*M_PI*200mm
// length_of_arc = (Rhvnd/360) * circum
// pixels_for_arc = (ccr->width/measured_seen_image_width) * length_of_arc
// oba* is the angle of the center of the object with respect to the
// left or top edge of the plane of the eye.
// cur* is the desired angle for the servo to center the object.
// See eye_to_object_in_image.odg for a diagram.
static int iseyes_move_eyes_to_object(meyec_t *ec, objectr_t *or, objectr_t *ol, camcap_t *ccr, camcap_t *ccl)
{
	double obarh, obalh, obarv, obalv;
	int currh, curlh, currv, curlv;

	double fvradius = sqrt(200*200+(141/2)*(141/2));
	double rwid = (ccr->width/141.0)*((Rhfvnd/360.0)*(2.0*M_PI*fvradius)); // arc width in pixels
	//double rwid = ((Rhfvnd/360)*(2*M_PI*fvradius)); // arc width in mm
	fvradius = sqrt(200*200+(135/2)*(135/2));
	double lwid = (ccl->width/135.0)*((Lhfvnd/360.0)*(2.0*M_PI*fvradius)); // arc width in pixels
	//double lwid = (Lhfvnd/360)*(2*M_PI*fvradius); // arc width in mm
	fvradius = sqrt(200*200+(106/2)*(106/2));
	double rhgt = (ccr->height/106.0)*((Rvfvnd/360.0)*(2.0*M_PI*fvradius)); // arc height in pixels
	fvradius = sqrt(200*200+(97/2)*(97/2));
	double lhgt = (ccl->height/97.0)*((Lvfvnd/360.0)*(2.0*M_PI*fvradius)); // arc height in pixels

	obarh = ((or->left+((or->right-or->left)/2))*(Rhfvnd/rwid))+(180-Rhfvnd)/2;
	currh = ec->e[MEYE_R].horiz_cur + (90.0 - obarh);
	obalh = ((ol->left+((ol->right-ol->left)/2))*(Lhfvnd/lwid))+(180-Lhfvnd)/2;
	curlh = ec->e[MEYE_L].horiz_cur + (90.0 - obalh);
	obarv = ((or->top+((or->bot-or->top)/2))*(Rvfvnd/rhgt))+(180-Rvfvnd)/2;
	currv = ec->e[MEYE_R].vert_cur + (90.0 - obarv);
	obalv = ((ol->top+((ol->bot-ol->top)/2))*(Lvfvnd/lhgt))+(180-Lvfvnd)/2;
	curlv = ec->e[MEYE_L].vert_cur + (obalv - 90);
	//curlv = ec->e[MEYE_L].vert_cur + (90.0 - obalv);
	printf("obarh=%f, currh=%d,obalh=%f, curlh=%d,obarv=%f, currv=%d,obalv=%f, curlv=%d\n", 
		obarh, currh, obalh, curlh, obarv, currv, obalv, curlv);
  
	if (meyec_move_eye_angle(ec, MEYE_R, MEYE_H, currh) == -1)
		return -1;
	if (meyec_move_eye_angle(ec, MEYE_R, MEYE_V, currv) == -1)
		return -1;
	if (meyec_move_eye_angle(ec, MEYE_L, MEYE_H, curlh) == -1)
		return -1;
	if (meyec_move_eye_angle(ec, MEYE_L, MEYE_V, curlv) == -1)
		return -1;

	// calc distance
	iseyes_calc_distance_to_object(ec, or, ol, ccr, ccl);
	
	return 0;
}

/*
For each object the right eye sees, see if the left eye sees the same object.
If it does, then do the calcs to find a solution for both eyes to rotate
horizontally and vertically to fixate on it. If there's no solution,
look for another instance in the left eye of the same object.
If there's no corresponding object or no solution then do nothing for now.
*/
static void iseyes_look(meyec_t *ec, objectr_t *or, objectr_t *ol, camcap_t *ccr, camcap_t *ccl)
{
	objectr_t *tor, *tol, *hightor, *hightol;
	float highprob = 0.0;

	printf("pre-calcs: right eye h,v=%d,%d, left eye y,v=%d,%d\n", 
			ec->e[MEYE_R].horiz_cur,
			ec->e[MEYE_R].vert_cur,
			ec->e[MEYE_L].horiz_cur,
			ec->e[MEYE_L].vert_cur);

	if (or == NULL || ol == NULL) {
		printf("One or both eyes didn't see any objects\n");
		return;
	}

	for (tor = or; tor != NULL; tor = tor->next) {
		for (tol = ol; tol != NULL; tol = tol->next) {
			if (strcmp(tor->name, tol->name) == 0) {
				printf("Both eyes see a %s, probs are %f and %f\n",
					tor->name, tor->prob, tol->prob);
				if (tor->prob + tol->prob > highprob) {
					highprob = tor->prob + tol->prob;
					hightor = tor;
					hightol = tol;
				}
			}
		}
	}
	if (highprob > 0.0) {
		printf("  chosen is %s\n", hightor->name);
		if (iseyes_move_eyes_to_object(ec, hightor, hightol, ccr, ccl) == -1) {
			return;
		}
#if FIND_COMMON_POINT
		if (iseyes_find_common_point(ec, tor, tol, ccr, ccl) == 1) {
			// look at the common point
			return;
		}
#endif
	}
}

int main(int argc, char **argv)
{
	meyec_t *ec;
	camcap_t *ccr, *ccl;
	brview_t *bv;
	brview_image_t *imgr, *imgl;
	unsigned imagesize;
	char *bufr, *bufl, **bufp;
	struct pollfd pfs[2];
	int nfds, i;
	objectr_t *objrsr = NULL, *objrsl = NULL;
	int eyesdone[2];

	if ((ec = meyec_new()) == NULL)
		exit(EXIT_FAILURE);
	if (meyec_look_straight_ahead(ec) == -1)
		exit(EXIT_FAILURE);	

	if ((bv = brview_setup(argc, argv)) == NULL) {
		fprintf(stderr, "brview_setup(): %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	if ((ccr = camcap_open("/dev/video1", NULL)) == NULL) {
		fprintf(stderr, "camcap_open(%s): %s\n", "/dev/video1", strerror(errno));
		exit(EXIT_FAILURE);
	}
	camcap_get_sizes(ccr, NULL, NULL, &imagesize);
	if ((bufr = malloc(imagesize)) == NULL)
		exit(EXIT_FAILURE);
	bufp = &bufr;
	if (camcap_init(ccr, (void **) bufp) == -1)
		exit(EXIT_FAILURE);

	if ((ccl = camcap_open("/dev/video0", NULL)) == NULL) {
		fprintf(stderr, "camcap_open(%s): %s\n", "/dev/video0", strerror(errno));
		exit(EXIT_FAILURE);
	}
	camcap_get_sizes(ccl, NULL, NULL, &imagesize);
	if ((bufl = malloc(imagesize)) == NULL)
		exit(EXIT_FAILURE);
	bufp = &bufl;
	if (camcap_init(ccl, (void **) bufp) == -1)
		exit(EXIT_FAILURE);

	int stride = brview_image_get_stride(ccr->width);
	unsigned char *datar = malloc(ccr->height * stride);
	memset(datar, 0xa0, ccr->height * stride); // start with a grey image
	if ((imgr = brview_image_create(bv, datar, 10, 10, ccr->width, ccr->height, stride)) == NULL) {
		fprintf(stderr, "brview_create_image(): %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}
	unsigned char *datal = malloc(ccl->height * stride);
	memset(datal, 0xa0, ccl->height * stride); // start with a grey image
	if ((imgl = brview_image_create(bv, datal, ccr->width+20, 10, ccl->width, ccl->height, stride)) == NULL) {
		fprintf(stderr, "brview_create_image(): %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}
#if 0
	// for testing
	int stride = brview_image_get_stride(8);
	unsigned char *data = malloc(9 * stride);
	memset(data, 0xa0, 9 * stride);
	if (brview_image_create(bv, data, 10, 10, 8, 9, stride) == -1) {
		fprintf(stderr, "brview_create_image(): %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}
#endif

	if (obrec_connect(&sockfd, &shmptr, sockname, shmname, ccr->width, ccr->height) == -1)
		exit(EXIT_FAILURE);

printf("start capture r\n");
	if (camcap_start_capturing(ccr) == -1)
		exit(EXIT_FAILURE);
printf("start capture l\n");
	if (camcap_start_capturing(ccl) == -1)
		exit(EXIT_FAILURE);
printf("done start captures\n");

	eyesdone[0] = eyesdone[1] = 0;
	while (1) {
		// poll fds, waiting for camera input
		pfs[0].fd = ccr->fd;
		pfs[1].fd = ccl->fd;
		pfs[0].events = pfs[1].events = POLLIN;
		if ((nfds = poll(pfs, 2, -1)) == -1) {
			perror("poll");
			exit(EXIT_FAILURE);
		}
		if (nfds == 0) {
			// this shouldn't happen since we don't have a timeout
			fprintf(stderr, "poll() returned 0\n");
			exit(EXIT_FAILURE);
		}
		for (i = 0; i < 2; i++) {
			if (pfs[i].revents & POLLIN) {
				printf("got a frame for /dev/video%d\n", i);
				// process frame
				if (i == 0) {
					printf("Right eye\n");
					if (camcap_read_frame(ccr) == -1)
						exit(EXIT_FAILURE);
					printf("read the frame\n");
					iseyes_convert_to_rgb(imgr, ccr);
					obrec_free_objectr_list(objrsr);
					obrec_find_objects_in_image(&objrsr, sockfd, shmptr, shmname, imgr);
					brview_image_update(imgr);
					if (camcap_requeue_frame(ccr) == -1)
						exit(EXIT_FAILURE);
					eyesdone[0] = 1;
				} else if (i == 1) {
					printf("Left eye\n");
					if (camcap_read_frame(ccl) == -1)
						exit(EXIT_FAILURE);
					printf("read the frame\n");
					iseyes_convert_to_rgb(imgl, ccl);
					obrec_free_objectr_list(objrsl);
					obrec_find_objects_in_image(&objrsl, sockfd, shmptr, shmname, imgl);
					brview_image_update(imgl);
					if (camcap_requeue_frame(ccl) == -1)
						exit(EXIT_FAILURE);
					eyesdone[1] = 1;
				}
			} else if (pfs[i].revents & POLLHUP ||
						pfs[i].revents & POLLERR) {
				fprintf(stderr, "poll returned problem with camera /dev/video%d\n", i);
				exit(EXIT_FAILURE);
			}
		}

		if (eyesdone[0] && eyesdone[1]) {
			printf("Looking\n");
			iseyes_look(ec, objrsr, objrsl, ccr, ccl);
			eyesdone[0] = eyesdone[1] = 0;
			if (stopcapturing) {
				int c;
printf("calling getchar()\n");
				c = getchar();
printf("c=0x%x\n", c);
				if (c == (int) 'p') {
					while ((c = getchar()) != (int) 'c')
						;
				}
				//pause();
			}
		}
	}
	if (camcap_stop_capturing(ccr) == -1)
		exit(EXIT_FAILURE);
	if (camcap_stop_capturing(ccl) == -1)
		exit(EXIT_FAILURE);
	camcap_close(ccr);
	camcap_close(ccl);

	meyec_destroy(ec);


	//iseyes_random_pans(ec);

	//while (1)
	//	iseyes_rectangular_pan(ec);


	//if (meyec_look_straight_ahead(ec) == -1)
	//	exit(EXIT_FAILURE);


	return EXIT_SUCCESS;
}

void iseyes_rectangular_pan(meyec_t *ec)
{
	double ha, va;

	va = 60;
	for (ha = 60; ha <= 120; ha++) {
		if (meyec_move_eyes_to_loc(ec, ha, va, 300) == -1)
			exit(EXIT_FAILURE);
		sleep(0.025);
	}
	for (va = 60; va <= 120; va++) {
		if (meyec_move_eyes_to_loc(ec, ha, va, 300) == -1)
			exit(EXIT_FAILURE);
		sleep(0.025);
	}
	for (; ha >= 60; ha--) {
		if (meyec_move_eyes_to_loc(ec, ha, va, 300) == -1)
			exit(EXIT_FAILURE);
		sleep(0.025);
	}
	for (; va >= 60; va--) {
		if (meyec_move_eyes_to_loc(ec, ha, va, 300) == -1)
			exit(EXIT_FAILURE);
		sleep(0.025);
	}
	if (meyec_move_eyes_to_loc(ec, 90, 90, 150) == -1)
		exit(EXIT_FAILURE);
}

void iseyes_random_pans(meyec_t *ec)
{
	double ha, va, hadest, vadest, dist, i, hainc, vainc;
	struct timespec sleeptime;

	srand(time(NULL));
	sleeptime.tv_sec = 0;

	ha = va = 90;
	while (1) {
		hadest = 50 + (((double) rand() / RAND_MAX) * 130);
		vadest = 50 + (((double) rand() / RAND_MAX) * 130);
		dist = sqrt(
					(abs(hadest-ha))*(abs(hadest-ha))+
					(abs(vadest-va))*(abs(vadest-va)));
		hainc = (hadest-ha)/dist;
		vainc = (vadest-va)/dist;
		sleeptime.tv_nsec = 9000000 + (((double) rand() / RAND_MAX) * 4000000);
	
//		printf("dist=%f, ha=%f, hadest=%f, hainc=%f, va=%f, vadest=%f, vainc=%f, st=%ld\n",
//			dist, ha, hadest, hainc, va, vadest, vainc, sleeptime.tv_nsec);
		for (i = 0; i < dist; i++) {
			if (meyec_move_eyes_to_loc(ec, ha, va, 300) == -1)
				exit(EXIT_FAILURE);
			nanosleep(&sleeptime, NULL);
			ha += hainc;
			va += vainc;
		}
		ha = hadest;
		va = vadest;
	}
}
